from x2cdict import VocabDict, PhraseDict
import click

@click.command()
@click.option("--word", help="Specify the word to look up", prompt="word")
@click.option("--pos", help="Specify PoS of the word", default=None)
@click.option("--fromlang", help="Specify the language of the word", prompt="translate from")
@click.option("--tolang", help="Specify the language the word to be translated into", prompt="translate to", default="cn")
@click.option("--external", help="Whether using the external source", prompt="using external source", type=bool, default=True)
def search_vocab(word, pos, fromlang, tolang, external):
  vd = VocabDict(fromlang, tolang)
  r = vd.search(word, pos, external)
  print(r)


@click.command()
@click.option("--word", help="Specify the word to look up", prompt="word")
@click.option("--fromlang", help="Specify the language of the word", prompt="translate from")
@click.option("--tolang", help="Specify the language the word to be translated into", prompt="translate to", default="cn")
@click.option("--external", help="Whether using the external source", prompt="using external source", type=bool, default=True)
def search_vocab_without_pos(word, fromlang, tolang, external):
  vd = VocabDict(fromlang, tolang)
  r = vd.search_without_pos(word, external)
  print(r)


@click.command()
@click.option("--phrase", help="Specify the phrase to look up", prompt="phrase")
@click.option("--fromlang", help="Specify the language of the word", prompt="translate from")
@click.option("--tolang", help="Specify the language the word to be translated into", prompt="translate to", default="cn")
def search_phrase(phrase, fromlang, tolang):
  vd = PhraseDict(fromlang, tolang)
  r = vd.search(phrase)
  print(r)




from googletrans import Translator

def reform_pos(pos):
  return pos.lower() + "." if pos != None else None

class GoogleVocabDict:

  def __init__(self, from_lang, to_lang="zh-cn"):
    self.from_lang, self.to_lang = from_lang, "zh-cn" if to_lang == "cn" else to_lang 
    self.api = Translator()

  def search(self, w, pos):
    try:
      _r = self.api.translate(w, src=self.from_lang, dest=self.to_lang)
      _pos = reform_pos(pos)
      result = {
        "word": w,
        "dict_pos": _pos,
        "meaning": _r.text,
        "extension": None,
        "variations": None,
        "from": "Google"
      }
    except Exception as e:
      result = None
    return result

class GooglePhraseDict:

  def __init__(self, from_lang, to_lang="zh-cn"):
    self.from_lang, self.to_lang = from_lang, "zh-cn" if to_lang == "cn" else to_lang 
    self.api = Translator()

  def search(self, p):
    try:
      _r = self.api.translate(p, src=self.from_lang, dest=self.to_lang)
      result = {
        "original": p,
        "translated": _r.text,
        "from": "Google"
      }
    except Exception as e:
      print(e)
      result = None
    return result





import deepl
import os

def reform_pos(pos):
  return pos.lower() + "." if pos != None else None


class DeepLVocabDict:
  def __init__(self, from_lang, to_lang="ZH"):
    self.from_lang, self.to_lang = from_lang.upper(), "ZH" if to_lang == "cn" else to_lang.upper() 
    self.api = deepl.Translator(os.getenv("DEEPL_AUTH_KEY"))

  def search(self, w, pos):
    try:
      _r = self.api.translate_text(w, source_lang=self.from_lang, target_lang=self.to_lang)
      _pos = reform_pos(pos)
      result = {
        "word": w,
        "dict_pos": _pos,
        "meaning": _r.text,
        "extension": None,
        "variations": None,
        "from": "DeepL"
      }
    except Exception as e:
      result = None
    return result


class DeepLPhraseDict:

  def __init__(self, from_lang, to_lang="ZH"):
    self.from_lang, self.to_lang = from_lang.upper(), "ZH" if to_lang == "cn" else to_lang.upper()
    self.api = deepl.Translator(os.getenv("DEEPL_AUTH_KEY"))

  def search(self, p):
    try:
      _r = self.api.translate_text(p, source_lang=self.from_lang, target_lang=self.to_lang)
      result = {
        "original": p,
        "translated": _r.text,
        "from": "DeepL"
      }
    except Exception as e:
      print(e)
      result = None
    return result





from x2cdict.db.db import DB
from x2cdict.posmap import POSMAP


class EN2CN(DB):
  """
  Encapsulation of mongodb
  """
  def __init__(self, user, password, host, authdb='admin'):
    self.dictname = 'en2cn'
    self.posmap = POSMAP['en']
    super().__init__(self.dictname, user, password, host, authdb)

  def search_vocab(self, text):
    result = self.vocabs.find_one({"word": text})
    if result != None:
      del result["_id"]
    return result 

  def search_sentences(self, vocab):
    result = list(self.sentences.find({"original": {"$regex": ".* "+vocab+" .*"}}, {"_id": 0}))[:2]
    return result

  def search(self, w, pos):
    result = None

    _pos = self.posmap[pos] if pos in self.posmap.keys() else [pos] 

    _r = self.search_vocab(w)
    if _r != None:
      ex = _r["explanation"]

      e = None # default: None 
      for i in ex: # Pick the fisrt, NOT CORRECT
        if i["pos"] in _pos:
          e = i
          break
 
      if e != None: 
        result = {
          "word": w,
          "dict_pos": e['pos'],
          "meaning": e['meaning'],
          "from": self.dictname 
        }
        result['variations'] = e['variations'] if 'variations' in e.keys() else None
        result['extension'] = e['extension'] if 'extension' in e.keys() else None
        result['examples'] = self.search_sentences(w)
    return result



from x2cdict.db import ES2CN, EN2CN, DE2CN
from x2cdict.lib import gen_dictname
import os

class QisheVocabDict:

  APIS = {
    "es2cn": ES2CN,
    "en2cn": EN2CN,
    "de2cn": DE2CN
  }

  def __init__(self, from_lang, to_lang="cn"):
    self.from_lang, self.to_lang = from_lang, to_lang
    self.dictname = gen_dictname(self.from_lang, self.to_lang) 

    self.api = None
    if self.dictname in self.__class__.APIS.keys():
      self.api = self.__class__.APIS[self.dictname](os.getenv("DICT_DB_USER", "dict"), os.getenv("DICT_DB_PASS", "turingmachine"), os.getenv("DICT_DB_HOST", "127.0.0.1"))

  def search_with_pos(self, w, pos):
    if self.api == None:
      return None
    return self.api.search(w, pos)

  def search_without_pos(self, w):
    if self.api == None:
      return None
    return self.api.search_vocab(w)



#from x2cdict.google import GoogleVocabDict, GooglePhraseDict
from x2cdict.qishe import QisheVocabDict
from x2cdict.deeplearning import DeepLVocabDict, DeepLPhraseDict

class VocabDict:
  """
  It combines qishe and deepl voacb api
  """

  def __init__(self, from_lang, to_lang="cn"):
    self.from_lang, self.to_lang = from_lang, to_lang
    self.deepl_api = DeepLVocabDict(from_lang, to_lang)
    self.qishe_api = QisheVocabDict(from_lang, to_lang)

  def search(self, w, pos, external=True):
    result = self.qishe_api.search_with_pos(w, pos)
    if result == None:
      result = self.deepl_api.search(w, pos) if external else None
    return result

  def search_without_pos(self, w, external=True):
    result = self.qishe_api.search_without_pos(w)
    if result == None:
      result = self.deepl_api.search(w, None) if external else None
    return result


class PhraseDict:
  """
  DeepL API to translate phrase 
  """

  def __init__(self, from_lang, to_lang="cn"):
    self.from_lang, self.to_lang = from_lang, to_lang
    self.deepl_api = DeepLPhraseDict(from_lang, to_lang)

  def search(self, p):
    result = self.deepl_api.search(p)
    return result



from x2cdict.db.db import DB
from x2cdict.posmap import POSMAP
import re

class DE2CN(DB):
  """
  Encapsulation of mongodb
  """
  def __init__(self, user, password, host, authdb='admin'):
    self.dictname = 'de2cn'
    self.posmap = POSMAP['de']
    super().__init__(self.dictname, user, password, host, authdb)
    self.verbs_extension = self.db.verbs_extension
    self.inverse_verbs_extension = self.db.inverse_verbs_extension

  def search_vocab(self, text):
    result = self.vocabs.find_one({"word": text})
    if result != None:
      del result['_id']
    return result 

  def search_verb_extension(self, text):
    result = self.verbs_extension.find_one({"word": text})
    return result 

  def search_verb_inverse_variation(self, text):
    result = self.inverse_verbs_extension.find_one({"word": text})
    return result 

  def verb_original(self, w, _pos):
    result = None
    verb = self.search_vocab(w)
    if verb != None:
      ex = verb["explanation"]

      e = None
      for i in ex:
        if i["pos"] in _pos:
          e = i
          break
 
      if e != None:
        result = {
          "word": w,
          "explanation": e,
          "variations": {
            "origin": w
          },
          "from": self.dictname 
        }

        vv = self.search_verb_extension(w) 
        if vv != None:
          result["extension"] = vv["extension"] 

      return result

  def verb_variation(self, w, _pos):
    result = None
    verb = self.search_verb_inverse_variation(w)
    if verb != None:
      original_verb = verb["variations"][0]["original"]
      info = self.verb_original(original_verb, _pos)
      if info != None:
        result = {
          "word": w,
          "explanation": info["explanation"],
          "variations": verb["variations"],
          "from": self.dictname
        }
        if "extension" in info.keys():
          result["extension"] = info["extension"]
    return result

  def other_vocab(self, w, _pos):
    result = None
    _r = self.search_vocab(w)
    if _r != None:
      ex = _r["explanation"]

      e = None # default: None 
      for i in ex:
        patterns = ["^"+_p.replace(".", "\.")+".*" for _p in _pos]
        flag = any([re.match(pattern, i["pos"]) for pattern in patterns])
        if flag:
          e = i
          break
      if e != None: 
        result = {
          "word": w,
          "explanation": e,
          "from": self.dictname 
        }
    return result

  def _search(self, w, pos):
    result = None

    _pos = self.posmap[pos] if pos in self.posmap.keys() else [pos] 

    if pos == "VERB" or pos == "AUX":
      result = self.verb_original(w, _pos)
      if result == None:
        result = self.verb_variation(w, _pos)
    else:
      result = self.other_vocab(w, _pos)
    return result

  def search(self, w, pos):
    result = self._search(w, pos)
    e = None
    if result != None:
      e = {}
      e['word'] = result['word']
      e['meaning'] = result['explanation']['meaning']
      e['dict_pos'] = result['explanation']['pos']
      e['from'] = result['from']
      if e['dict_pos'] not in self.posmap['VERB']:
        e['extension'] = result['explanation']['extension'] if 'extension' in result['explanation'].keys() else None
        e['variations'] = None
      else:
        e['extension'] = result['extension'] if 'extension' in result.keys() else None
        e['variations'] = result['variations'] if 'variations' in result.keys() else None
      #e['examples'] = self.search_sentences(w)
    return e



from pymongo import MongoClient
from urllib.parse import quote_plus

class DB:
  """
  Encapsulation of mongodb
  """
  def __init__(self, dbname, user, password, host, authdb='admin'):
    uri = "mongodb://{}:{}@{}/{}".format(quote_plus(user), quote_plus(password), host, authdb)
    self.client = MongoClient(uri)
    self.db = self.client[dbname]
    self.vocabs = self.db.vocabs if dbname!= "de2cn" else self.db.base
    self.sentences = self.db.sentences

